#include <AccelStepper.h>
#include <MultiStepper.h>
#define stepPin1 2
#define dirPin1 3
#define stepPin2 4
#define dirPin2 5
#define motorInterfaceType 1
#include <SoftwareSerial.h>   // Incluimos la librería  SoftwareSerial  
SoftwareSerial BT(10, 11);   // Definimos los pines TX y RX del Arduino conectados al Bluetooth

// Create a new instance of the AccelStepper class:
AccelStepper stepper1 = AccelStepper(motorInterfaceType, stepPin1, dirPin1);
AccelStepper stepper2 = AccelStepper(motorInterfaceType, stepPin2, dirPin2);
#define EXTRACT 0
#define INJECT 1
bool actionstepper1 = 0;
float positionstepper1 = 5.7, positionstepper2 = 5.7;
//Volume of each pixel (in mL)
float volume = 0.3;
//Distance between mL in the srynges (mm)
const float distancemL = 57;
//Max and min position of steppers
const int posmax = 53, posmin = 0;
const int stepsperrevolution = 400, lead = 5, acceleration = 1000;
const float stepstoadvance = stepsperrevolution / lead;
const float stepspermL = stepstoadvance * distancemL;
//Volume control mL
float volumesyringe1 = 1;
float volumesyringe2 = 1;
float volumetotal = 0;
int velocity = 1000;
//Time control ms
float timetochange = 500;

#include <Servo.h>
//Define states of servos/valves
const int OPEN = 90;
const int CLOSE = 110;
const int OPEN29 = 80;
const int CLOSE29 = 100;
const int OPEN25 = 92;
const int CLOSE25 = 112;
const int OPEN27 = 85;
const int CLOSE27 = 105;


// Define the number of pixels
const int NPIXELS =  16;
// Create the servo objects.
Servo servoStepper1[NPIXELS];
Servo servoStepper2[NPIXELS];
int servo_pins[NPIXELS * 2];

#define stateSETUP 1
#define stateREADCHANGES 2
#define statePREPARATION 3
#define stateMOVEVALVES 4
#define stateMOVESTEPPER 5
int state;
boolean resetstring = true;
int rows, columns, PtC, newstates;
int** screen;
int* xchange, * ychange, * newstatearray;
const byte numChars = 128;
char receivedChars[numChars]; // an array to store the received data
boolean bluetooth = false;

void setup() {
  BT.begin(9600);       // Inicializamos el puerto serie BT que hemos creado
  Serial.begin(9600);
  stepper1.setMaxSpeed(velocity);
  stepper1.setAcceleration(acceleration);
  stepper1.setSpeed(velocity);
  stepper2.setMaxSpeed(velocity);
  stepper2.setAcceleration(acceleration);
  stepper2.setSpeed(velocity);

  for (int i = 22; i - 22 < NPIXELS * 2; i++) {
    if (i <= 53) {
      // Creates servo pins
      servo_pins[i - 22] = i;
    }
    else {
      Serial.println("There are not enought pins, please reduce screen size");
    }
  }
  // Attach servos to digital pins on the Arduino
  int j = 0;
  for (int i = 0; i < NPIXELS; i++) {
    // Attach the servo to the servo object
    servoStepper1[i].attach(servo_pins[(j)]);
    // Serial.print(servo_pins[j]);Serial.print(i);Serial.println("Stepper 1");
    j++;
    servoStepper2[i].attach(servo_pins[j]);
    // Serial.print(servo_pins[j]);Serial.print(i);Serial.println("Stepper 2");
    j++;
    servoStepper1[i].write(CLOSE);
    servoStepper2[i].write(CLOSE);
  }
  // for (int i = 0; i < NPIXELS * 2; i++) {Serial.println(servo_pins[i]);}
  state = stateSETUP;
}

void loop() {
  switch (state) {
    case stateSETUP:
      Serial.println("SETUP");
      BT.println("SETUP");
      SETUP();
      break;
    case stateREADCHANGES:
      Serial.println("LEER CAMBIOS");
      BT.println("LEER CAMBIOS");
      READCHANGES();
      break;
    case statePREPARATION:
      Serial.println("PREPARACION-ELEGIR MOTOR");
      BT.println("PREPARACION-ELEGIR MOTOR");
      CHOOSESTEPPER();
      Serial.println("PREPARACION-MOVER VALVULAS");
      BT.println("PREPARACION-MOVER VALVULAS");
      MOVEVALVES();
      state = stateMOVESTEPPER;
      break;
    case stateMOVESTEPPER:
      Serial.println("MOVER MOTOR");
      BT.println("MOVER MOTOR");
      MOVESTEPPER();
      break;
    default:
      // statements
      break;
  }

} //END OF PROGRAM

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////FUNCTIONS////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////

//READS THE INPUT IN SERIAL MONITOR. OUTPUT IS A CHAR
void recvWithEndMarker() {
  while (1) {
    if (resetstring == true)
    {
      memset (receivedChars, '\0', sizeof(receivedChars));
      resetstring = false;
    }
    static byte ndx = 0;
    char endMarker = '\n';
    char rc;

    while (BT.available() > 0 || Serial.available() > 0) {
      if (Serial.available() > 0)
      {
        rc = Serial.read();
        bluetooth = false;
      }
      if (BT.available() > 0)
      {
        rc = BT.read();
        bluetooth = true;
      }

      if (rc != endMarker) {
        receivedChars[ndx] = rc;
        ndx++;
        if (ndx >= numChars) {
          ndx = numChars - 1;
        }
      }
      else {
        receivedChars[ndx] = '\0'; // terminate the string
        ndx = 0;
        return;
      }
    }
  }
}

//SHOWS CHAR RECEIVED WITH "recvWithEndMarker()"
void showNewData() {
  while (1) {
    switch (state) {
      case stateSETUP:
        Serial.print("INPUT: ");
        Serial.println(receivedChars);
        resetstring = true;
        break;

      case stateREADCHANGES:
        Serial.print("INPUT: ");
        Serial.println(receivedChars);
        resetstring = true;
        break;

      case statePREPARATION:
        resetstring = true;
        break;

      case stateMOVEVALVES:
        resetstring = true;
        break;

      case stateMOVESTEPPER:
        resetstring = true;
        break;

      default:
        // statements
        break;
    }
    return;
  }

}

//Converts char received from one position to the next space or end to an int
int readInput (char chain[], int* const readPosition) {
  int number, j = 0, lenght, total = 0;
  bool first = false;
  char part[numChars];
  if (first == false);
  {
    while (chain[*readPosition] == ' ')
    {
      (*readPosition)++;
    }
    first = true;
  }
  while (chain[*readPosition] != ' ' && chain[*readPosition] != '\0')
  {
    part[j] = chain[*readPosition];
    (*readPosition)++;
    j++;
  }
  lenght = j;
  for (int a = 0; a < lenght; a++)
  {
    number = part[a] - '0';
    for (int b = 1; b < (lenght - a); b++)
    {
      number = number * 10;
    }
    total = number + total;
  }
  return total;
}

//CREATES A 2D MATRIX OF N SPACES (N DEFINED BY USER THROUGH SERIAL MONITOR
int** matrixCreation(int matrix_width, int matrix_height) {
  int** matrix = new int*[matrix_width];
  for ( int i = 0; i < matrix_width; ++i )
  {
    matrix[i] = new int[matrix_height];
  }
  return matrix;
}

//COUNTS SPACES IN CHAR RECEIVED AND INFERES NUMBER OF PIXELS TO CHANGE
int countPixelsToChange(char pixelsString[])
{
  int Position = 0;
  int numberofpixels = 1 ;
  while (pixelsString[Position] == ' ')
  {
    Position ++;
  }
  while (pixelsString[Position] != '\0')
  {
    if (pixelsString[Position] == ' ')
    {
      numberofpixels++;
    }
    Position++;
  }
  return numberofpixels ;
}

//CREATES AN ARRAY OF PIXEL TO CHANGE AND FILL IT WITH THE INFORMATION
int* pixelChangeArray(int amountpixels, char axis, bool bluetooth) {
  int* newarray = new int[amountpixels];
  int readPosition = 0;
  while (receivedChars[readPosition] == ' ')
  {
    //    Serial.println(receivedChars[readPosition]);
    //    Serial.println("Saltando espacio ");
    readPosition ++;
  }
  for ( int i = 0; i < amountpixels; ++i ) //Fill it
  {
    if (bluetooth == true)
    {
      switch (axis) {
        case 'x':
          newarray[i] = readInput(receivedChars, &readPosition);
          readPosition++;
          readInput(receivedChars, &readPosition);
          readPosition++;
          readInput(receivedChars, &readPosition);
          readPosition++;
          break;
        case 'y':
          readInput(receivedChars, &readPosition);
          readPosition++;
          newarray[i] = readInput(receivedChars, &readPosition);
          readPosition++;
          readInput(receivedChars, &readPosition);
          readPosition++;
          break;
        case 's':
          readInput(receivedChars, &readPosition);
          readPosition++;
          readInput(receivedChars, &readPosition);
          readPosition++;
          newarray[i] = readInput(receivedChars, &readPosition);
          readPosition++;
          break;
      }
    }
    if (bluetooth == false)
    {
      switch (axis) {
        case 'x':
          newarray[i] = readInput(receivedChars, &readPosition);
          readPosition++;
          readInput(receivedChars, &readPosition);
          readPosition++;
          break;
        case 'y':
          readInput(receivedChars, &readPosition);
          readPosition++;
          newarray[i] = readInput(receivedChars, &readPosition);
          readPosition++;
          break;
        case 's':
          newarray[i] = readInput(receivedChars, &readPosition);
          readPosition++;
          break;
      }
    }
  }
  return newarray; //Borrar tras usar en programa principal
}

void moveSERVO(int charposition) {
  switch (actionstepper1) {
    case INJECT:
      if (newstatearray[charposition] == 1) //New state requires filling, valves of stepper 1 will open and stepper 2 will close
      {
        servoStepper1[(ychange[charposition]*columns + xchange[charposition])].write(OPEN);
        servoStepper2[(ychange[charposition]*columns + xchange[charposition])].write(CLOSE);
        if ((ychange[charposition]*columns + xchange[charposition]) == 3)
        {
          servoStepper2[(ychange[charposition]*columns + xchange[charposition])].write(CLOSE29);
        }
        if ((ychange[charposition]*columns + xchange[charposition]) == 1)
        {
          servoStepper2[(ychange[charposition]*columns + xchange[charposition])].write(CLOSE25);
        }
         if ((ychange[charposition]*columns + xchange[charposition]) == 2)
        {
          servoStepper2[(ychange[charposition]*columns + xchange[charposition])].write(CLOSE27);
        }
        //                Serial.println("New state requires filling, valves of stepper 1 will open and stepper 2 will close");
        //                Serial.print("Fill Pixel number ");Serial.print((ychange[charposition]*columns+xchange[charposition]));
        //                Serial.print(" in coordinates ");Serial.print(xchange[charposition]);Serial.print(" ");Serial.println(ychange[charposition]);
        //                Serial.println("The Pins used are: "); Serial.print("  Stepper 1 Inject (OPEN) "); Serial.println(servo_pins[(ychange[charposition]*columns + xchange[charposition]) * 2]);
        //                Serial.print("  Stepper 2 Extract (CLOSE) "); Serial.println(servo_pins[1 + (ychange[charposition]*columns + xchange[charposition]) * 2]);Serial.print("\n");
        //                screen[xchange[charposition]][ychange[charposition]] = 1;
      }
      if (newstatearray[charposition] == 0) //New state requires emptying, valves of stepper 1 will close and stepper 2 will open
      {
        servoStepper1[(ychange[charposition]*columns + xchange[charposition])].write(CLOSE);
        servoStepper2[(ychange[charposition]*columns + xchange[charposition])].write(OPEN);
        if ((ychange[charposition]*columns + xchange[charposition]) == 3)
        {
          servoStepper2[(ychange[charposition]*columns + xchange[charposition])].write(OPEN29);
        }
        if ((ychange[charposition]*columns + xchange[charposition]) == 1)
        {
          servoStepper2[(ychange[charposition]*columns + xchange[charposition])].write(OPEN25);
        }
         if ((ychange[charposition]*columns + xchange[charposition]) == 2)
        {
          servoStepper2[(ychange[charposition]*columns + xchange[charposition])].write(OPEN27);
        }
        //                Serial.println("New state requires emptying, valves of stepper 1 will close and stepper 2 will open");
        //                Serial.print("Empty Pixel number ");Serial.print((ychange[charposition]*columns+xchange[charposition]));
        //                Serial.print(" in coordinates ");Serial.print(xchange[charposition]);Serial.print(" ");Serial.println(ychange[charposition]);
        //                Serial.println("The Pins used are: "); Serial.print("  Stepper 1 Inject (CLOSE) "); Serial.println(servo_pins[(ychange[charposition]*columns + xchange[charposition]) * 2]);
        //                Serial.print("  Stepper 2 Extract (OPEN) "); Serial.println(servo_pins[1 + (ychange[charposition]*columns + xchange[charposition]) * 2]);Serial.print("\n");
      }
      break;
    case EXTRACT:
      if (newstatearray[charposition] == 1) //New state requires filling, valves of stepper 2 will open and stepper 1 will close
      {
        servoStepper1[(ychange[charposition]*columns + xchange[charposition])].write(CLOSE);
        servoStepper2[(ychange[charposition]*columns + xchange[charposition])].write(OPEN);
        if ((ychange[charposition]*columns + xchange[charposition]) == 3)
        {
          servoStepper2[(ychange[charposition]*columns + xchange[charposition])].write(OPEN29);
        }
        if ((ychange[charposition]*columns + xchange[charposition]) == 1)
        {
          servoStepper2[(ychange[charposition]*columns + xchange[charposition])].write(OPEN25);
        }
         if ((ychange[charposition]*columns + xchange[charposition]) == 2)
        {
          servoStepper2[(ychange[charposition]*columns + xchange[charposition])].write(OPEN27);
        }
        //                Serial.println("New state requires filling, valves of stepper 2 will open and stepper 1 will close");
        //                Serial.print("Fill Pixel number ");Serial.print((ychange[charposition]*columns+xchange[charposition]));
        //                Serial.print(" in coordinates ");Serial.print(xchange[charposition]);Serial.print(" ");Serial.println(ychange[charposition]);
        //                Serial.println("The Pins used are: "); Serial.print("  Stepper 1 Extract (CLOSE) "); Serial.println(servo_pins[(ychange[charposition]*columns + xchange[charposition]) * 2]);
        //                Serial.print("  Stepper 2 Inject (OPEN) "); Serial.println(servo_pins[1 + (ychange[charposition]*columns + xchange[charposition]) * 2]);Serial.print("\n");
      }
      if (newstatearray[charposition] == 0) //New state requires emptying, valves of stepper 2 will close and stepper 1 will open
      {
        servoStepper1[(ychange[charposition]*columns + xchange[charposition])].write(OPEN);
        servoStepper2[(ychange[charposition]*columns + xchange[charposition])].write(CLOSE);
        if ((ychange[charposition]*columns + xchange[charposition]) == 3)
        {
          servoStepper2[(ychange[charposition]*columns + xchange[charposition])].write(CLOSE29);
        }
        if ((ychange[charposition]*columns + xchange[charposition]) == 1)
        {
          servoStepper2[(ychange[charposition]*columns + xchange[charposition])].write(CLOSE25);
        }
         if ((ychange[charposition]*columns + xchange[charposition]) == 2)
        {
          servoStepper2[(ychange[charposition]*columns + xchange[charposition])].write(CLOSE27);
        }
        //                Serial.println("New state requires filling, valves of stepper 1 will open and stepper 2 will close");
        //                Serial.print("Empty Pixel number ");Serial.print((ychange[charposition]*columns+xchange[charposition]));
        //                Serial.print(" in coordinates ");Serial.print(xchange[charposition]);Serial.print(" ");Serial.println(ychange[charposition]);
        //                Serial.println("The Pins used are: "); Serial.print("  Stepper 1 Extract (OPEN) "); Serial.println(servo_pins[(ychange[charposition]*columns + xchange[charposition]) * 2]);
        //                Serial.print("  Stepper 2 Extract (CLOSE) "); Serial.println(servo_pins[1 + (ychange[charposition]*columns + xchange[charposition]) * 2]);Serial.print("\n");
      }
      break;
  }
  return;
}
void moveSTEPPER(int stepper, int pulses)
{
  //  Serial.print("Starting movement "); Serial.print("Stepper "); Serial.print(stepper); Serial.print(" Pulses "); Serial.println(pulses);
  switch (stepper)
  {
    case 1:
      stepper1.setCurrentPosition(0);
      stepper1.moveTo(pulses);
      while (stepper1.distanceToGo() != 0)
      {
        stepper1.run();
        //Serial.println("Moving stepper 1 one unit");
      }
      break;
    case 2:
      stepper2.setCurrentPosition(0);
      stepper2.moveTo(pulses);
      while (stepper2.distanceToGo() != 0)
      {
        stepper2.run();
        //Serial.println("Moving stepper 2 one unit");
      }
      break;
  }
}
void freeSTRINGS()
{
  delete[] xchange; delete[] ychange; delete[] newstatearray;
  return;
}
void compareChanges()
{
  bool single = 1;
  int j = 0;
  for (int i = 0; i < PtC; i++)
  {
    if (newstatearray[i] == 2 || newstatearray[i] == 3)
    {

    }
    else
    {
      for (int k = i; k < PtC; k++)
      {

        if (xchange[k] == xchange[i] && ychange[k] == ychange[i])
        {
          single = 0;
        }
        if (single == 0 && screen[xchange[k]][ychange[k]] != newstatearray[i] )
        {
          xchange[j] = xchange[k];
          ychange[j] = ychange[k];
          newstatearray[j] = newstatearray[k];
          newstatearray[k] = newstatearray[k] + 2;
        }
        single = 1;
      }
      if (screen[xchange[j]][ychange[j]] != newstatearray[j])
      {
        j++;
      }
    }
  }
  PtC = j;
  for (int i = 0; i < PtC; i++)
  {
    if (newstatearray[i] == 2 ||  newstatearray[i] == 3)
    {
      newstatearray[i] = newstatearray[i] - 2;
    }
  }
  newstates = j;
}

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////BIGGER STATES///////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void SETUP()
{
  Serial.println("Introduce screen size (Rows Columns)");
  BT.println("Introduce screen size (Rows Columns)");
  recvWithEndMarker();
  showNewData();
  if (countPixelsToChange(receivedChars) != 2 )
  {
    Serial.println("Please introduce two positive numbers");
    BT.println("Please introduce two positive numbers");
    return;
  }
  int readPosition = 0;
  rows = readInput(receivedChars, &readPosition);
  readPosition++;
  columns = readInput(receivedChars, &readPosition);
  //  Serial.print(rows);Serial.print(" ");Serial.println(columns);
  if (rows * columns > NPIXELS)
  { Serial.println("There are not enought pins, please reduce screen size");
    return;
  }
  if (rows <= 0 || columns <= 0 )
  {
    Serial.println("Please introduce two positive numbers");
    BT.println("Please introduce two positive numbers");
    return;
  }
  //    Serial.print("There are "); Serial.print(rows); Serial.print(" rows and "); Serial.print(columns); Serial.println(" columns. ");
  screen = matrixCreation(rows, columns);
  for (int i = 0; i < rows; i++)
  {
    for (int j = 0; j < columns; j++)
    {
      screen[i][j] = 0;
    }
  }
  state = stateREADCHANGES;
}
void READCHANGES()
{
  //Receive char x0 y0 x1 y1...and count number of pixels
  Serial.println("Introduce pixels to change");
  BT.println("Introduce pixels to change");
  recvWithEndMarker();
  showNewData();
  if (bluetooth == true)
  {
    if (countPixelsToChange(receivedChars) % 3 != 0)
    {
      Serial.println("One pixel coordinate is missing, please introduce pixels again");
      return;
    }
    PtC = countPixelsToChange(receivedChars) / 3;
    //  Serial.print("You want to change "); Serial.print(PtC); Serial.println(" pixels ");
    //Converts received char to int array of x and y coordinates
    xchange = pixelChangeArray(PtC, 'x', bluetooth);
    ychange = pixelChangeArray(PtC, 'y', bluetooth);
    newstatearray = pixelChangeArray(PtC, 's', bluetooth);
    newstates = countPixelsToChange(receivedChars) / 3;
  }
  if (bluetooth == false)
  {
    if (countPixelsToChange(receivedChars) % 2 != 0)
    {
      Serial.println("One pixel coordinate is missing, please introduce pixels again");
      return;
    }
    PtC = countPixelsToChange(receivedChars) / 2;
    //  Serial.print("You want to change "); Serial.print(PtC); Serial.println(" pixels ");
    //Converts received char to int array of x and y coordinates
    xchange = pixelChangeArray(PtC, 'x', bluetooth);
    ychange = pixelChangeArray(PtC, 'y', bluetooth);
    //Receive char 0 1 1 0...of pixel new states
    Serial.println("Introduce pixels state");
    recvWithEndMarker();
    showNewData();
    newstates = countPixelsToChange(receivedChars);
    newstatearray = pixelChangeArray(newstates, 's', bluetooth);
  }
  for (int i = 0; i < PtC; i++)
  {
    if (xchange[i] >= rows || ychange[i] >= columns)
    {
      Serial.print("The pixel number: "); Serial.print(i);
      Serial.print(" in coordinates: "); Serial.print(xchange[i]); Serial.print(" "); Serial.print(ychange[i]);
      Serial.println(" is not within screen size, please introduce pixels again");
      return;
    }
  }

  for (int i = 0; i < newstates; i++)
  {
    if (newstatearray[i] != 0 && newstatearray[i] != 1)
    {
      Serial.print("The state of pixel number: "); Serial.print(i); Serial.println(" is not 0 or 1, please introduce pixels again");
      return;
    }
  }
  if (PtC != newstates)
  {
    Serial.println("The number of pixels and states don't match, please introduce pixels again");
    return;
  }
  //  Serial.println("Comparando cambios...");
  compareChanges();
  Serial.println("New pixels coordinates are ");
  BT.println("New pixels coordinates are ");
  for (int i = 0; i < PtC; i++)
  {
    Serial.print(xchange[i]); Serial.print(" "); Serial.print(ychange[i]); Serial.println(" ");
    BT.print(xchange[i]); BT.print(" "); BT.print(ychange[i]); BT.println(" ");
  }
  Serial.println("Pixels states are");
  BT.println("Pixels states are");
  for (int i = 0; i < newstates; i++)
  {
    Serial.print(newstatearray[i]);
    BT.print(newstatearray[i]);
    Serial.print(" ");
    BT.print(" ");
  }
  Serial.print("\n"); Serial.print("\n");
  BT.print("\n"); BT.print("\n");
  //ADDITIONAL INFO
  ////   Volume of each pixel in mL
  //    Serial.println("Introduce Volume of each pixel in mL*1000");
  //   BT.println("Introduce Volume of each pixel in mL*1000");
  //    recvWithEndMarker();
  //    showNewData();
  //    int readPosition = 0;
  //    volume = readInput(receivedChars, &readPosition);
  //  //  volume = volume/1000;
  //    Serial.println(volume);
  //   BT.println(volume);

  Serial.println("Introduce time to change in ms");
  BT.println("Introduce time to change in ms");
  recvWithEndMarker();
  showNewData();
  Serial.println(receivedChars);
  int readPosition = 0;
  timetochange = readInput(receivedChars, &readPosition);
  Serial.println(timetochange);
  BT.println(timetochange);

  state = statePREPARATION;
  return;
}
void CHOOSESTEPPER()
{
  //  Serial.print("The position of stepper 1 is "); Serial.println(positionstepper1);
  //  Serial.print("The position of stepper 2 is "); Serial.println(positionstepper2);
  //  Serial.print("Therefore ");
  if (positionstepper1 >= positionstepper2)
  {
    //    Serial.print("stepper 1 will extract and stepper 2 will inject ");
    actionstepper1 = EXTRACT;
  }
  else
  {
    //    Serial.println("stepper 1 will inject and stepper 2 will extract ");
    actionstepper1 = INJECT;
  }
  return;
}

void MOVEVALVES()
{
  Serial.println("Moving servos");
  for (int i = 0; i < PtC; i++)
  {
    //    Serial.print("Screen state is: "); Serial.print(screen[xchange[i]][ychange[i]]); Serial.print(" New state is "); Serial.println(newstatearray[i]);
    moveSERVO(i);
  }
  delay(500);
  return;
}
void MOVESTEPPER()
{
  float pixelstofill, pixelstoempty;
  float pulsestofill, pulsestoempty;
  float finalpos1, finalpos2;
  float speedempty, speedfill;

  //Number of pixels to fill and empty
  pixelstofill = 0;
  pixelstoempty = 0;
  pulsestofill = 0;
  pulsestoempty = 0;
  for (int i = 0; i < newstates; i++)
  {
    pixelstofill = pixelstofill + newstatearray[i];
  }
  pixelstoempty = newstates - pixelstofill;
  Serial.print("Pixels to fill: "); Serial.println(pixelstofill);
  Serial.print("Pixels to empty: "); Serial.println(pixelstoempty);
  pulsestofill = pixelstofill * stepspermL * volume ;
  pulsestoempty = pixelstoempty * stepspermL * volume ;
  if (volumesyringe1 + volumesyringe2 <= volumetotal + ( pixelstofill * volume) - (pixelstoempty * volume))
  {
    Serial.print("Each pixel volume is "); Serial.println(volume);
    Serial.print("Expected final volume is "); Serial.print(volumetotal + ( pixelstofill * volume) - (pixelstoempty * volume)); Serial.print(" of "); Serial.println(volumesyringe1 + volumesyringe2);
    Serial.println("There is not enought volume. Introduce data again.");
    for (int i = 0; i < NPIXELS; i++) {
      if (servoStepper1[i].read() == OPEN)
      {
        servoStepper1[i].write(CLOSE);
      }
      if (servoStepper2[i].read() == OPEN)
      {
        servoStepper2[i].write(CLOSE);
      }
       if (servoStepper2[i].read() == OPEN25)
      {
        servoStepper2[i].write(CLOSE25);
      }
      if (servoStepper2[i].read() == OPEN29)
      {
        servoStepper2[i].write(CLOSE29);
      }
       if (servoStepper2[i].read() == OPEN27)
      {
        servoStepper2[i].write(CLOSE27);
      }

    }
    freeSTRINGS();
    state = stateREADCHANGES;
    return;
  }

  Serial.print("Pulses to fill: "); Serial.println(pulsestofill);
  Serial.print("Pulses to empty: "); Serial.println(pulsestoempty);

  speedfill = (pulsestofill / timetochange * 1000);
  speedempty = (pulsestoempty / timetochange * 1000);
  if (speedfill > 1000)
  { Serial.print("Not possible, min time is ");
    Serial.print(pulsestofill); Serial.println(" ms with 1000 steps/s. Using that speed.");
    speedfill = 1000;
  }
  if (speedempty > 1000)
  { Serial.print("Not possible, min time is ");
    Serial.print(pulsestoempty); Serial.println(" ms with 1000 steps/s. Using that speed.");
    speedempty = 1000;
  }
  //  Serial.println(stepstoadvance);
  //  Serial.println("Moviendo....");
  while (1)
  {
    int contador = 0;
    int contador2 = 0;
    switch (actionstepper1)
    {
      case INJECT:
        stepper1.setMaxSpeed(speedfill);
        stepper2.setMaxSpeed(speedempty);
        //        Serial.println(stepper1.speed());
        //        Serial.println(stepper2.speed());
        finalpos1 = positionstepper1 + pulsestofill / stepstoadvance;
        finalpos2 = positionstepper2 - pulsestoempty / stepstoadvance;
        if (finalpos1 > posmax || posmin > finalpos2)
        {
          Serial.println("There is not enought travel. Steppers will switch midway.");
        }
        Serial.println("Injecting stepper 1. Extracting stepper 2.");

        //Both stepper must go to the limit
        if (finalpos1 > posmax && posmin > finalpos2)
        {
          //          stepper1.setCurrentPosition(0);
          stepper1.move(-((posmax - positionstepper1) * stepspermL));
          //          stepper2.setCurrentPosition(0);
          stepper2.move((positionstepper2 - posmin) * stepspermL);
          while (stepper1.distanceToGo() != 0 || stepper2.distanceToGo() != 0)
          {
            stepper1.run();
            stepper2.run();
            //            if (stepper1.run() == true)
            //            {
            //              contador++;
            //              Serial.print("Pulse "); Serial.print(contador); Serial.println(" of stepper 1 ");
            //            }
            //            if (stepper2.run() == true)
            //            {
            //              contador2++;
            //              Serial.print("Pulse "); Serial.print(contador2); Serial.println(" of stepper 2 ");
            //            }
          }
          Serial.print("El motor 1 se ha movido "); Serial.print((posmax - positionstepper1) * stepspermL); Serial.println(" pulsos ");
          Serial.print("El motor 2 se ha movido "); Serial.print((positionstepper2 - posmin) * stepspermL); Serial.println(" pulsos ");
          pulsestofill = pulsestofill - (posmax - positionstepper1) * stepspermL;
          pulsestoempty = pulsestoempty - (positionstepper2 - posmin) * stepspermL;
          positionstepper1 = posmax;
          positionstepper2 = posmin;
        }

        //Only stepper 1 move to max limit
        if (finalpos1 > posmax && finalpos2 >= posmin)
        {
          //          stepper1.setCurrentPosition(0);
          stepper1.move(-((posmax - positionstepper1) * stepspermL));
          //          stepper2.setCurrentPosition(0);
          stepper2.move(pulsestoempty);
          while (stepper1.distanceToGo() != 0 || stepper2.distanceToGo() != 0)
          {
            stepper1.run();
            stepper2.run();
            //            if (stepper1.run() == true)
            //            {
            //              contador++;
            //              Serial.print("Pulse "); Serial.print(contador); Serial.println(" of stepper 1 ");
            //            }
            //            if (stepper2.run() == true)
            //            {
            //              contador2++;
            //              Serial.print("Pulse "); Serial.print(contador2); Serial.println(" of stepper 2 ");
            //            }
          }
          Serial.print("El motor 1 se ha movido "); Serial.print((posmax - positionstepper1) * stepspermL); Serial.println(" pulsos ");
          Serial.print("El motor 2 se ha movido "); Serial.print(pulsestoempty); Serial.println(" pulsos ");
          pulsestofill = pulsestofill - (posmax - positionstepper1) * stepspermL;
          pulsestoempty = 0;
          positionstepper1 = posmax;
          positionstepper2 = finalpos2;
        }

        //Only stepper 2 move to max limit
        if (posmax >= finalpos1 && posmin > finalpos2)
        {
          //          stepper1.setCurrentPosition(0);
          stepper1.move(-pulsestofill);
          //          stepper2.setCurrentPosition(0);
          stepper2.move((positionstepper2 - posmin ) * stepspermL);
          while (stepper1.distanceToGo() != 0 || stepper2.distanceToGo() != 0)
          {
            stepper1.run();
            stepper2.run();
            //            if (stepper1.run() == true)
            //            {
            //              contador++;
            //              Serial.print("Pulse "); Serial.print(contador); Serial.println(" of stepper 1 ");
            //            }
            //            if (stepper2.run() == true)
            //            {
            //              contador2++;
            //              Serial.print("Pulse "); Serial.print(contador2); Serial.println(" of stepper 2 ");
            //            }
          }
          Serial.print("El motor 1 se ha movido "); Serial.print(pulsestofill); Serial.println(" pulsos ");
          Serial.print("El motor 2 se ha movido "); Serial.print((positionstepper2 - posmin) * stepspermL); Serial.println(" pulsos ");
          pulsestofill = 0;
          pulsestoempty = pulsestoempty - ((positionstepper2 - posmin) * stepspermL);
          positionstepper1 = finalpos1;
          positionstepper2 = posmin;
        }
        //Both steppers inside range
        if (posmax >= finalpos1 && finalpos2 >= posmin)
        {
          //          stepper1.setCurrentPosition(0);
          stepper1.move(-pulsestofill);
          //          stepper2.setCurrentPosition(0);
          stepper2.move(pulsestoempty);
          while (stepper1.distanceToGo() != 0 || stepper2.distanceToGo() != 0)
          {
            stepper1.run();
            stepper2.run();
            //            if (stepper1.run() == true)
            //            {
            //              contador++;
            //              Serial.print("Pulse "); Serial.print(contador); Serial.println(" of stepper 1 ");
            //            }
            //            if (stepper2.run() == true)
            //            {
            //              contador2++;
            //              Serial.print("Pulse "); Serial.print(contador2); Serial.println(" of stepper 2 ");
            //            }
          }
          Serial.print("El motor 1 se ha movido "); Serial.print(pulsestofill); Serial.println(" pulsos ");
          Serial.print("El motor 2 se ha movido "); Serial.print(pulsestoempty); Serial.println(" pulsos ");
          pulsestofill = 0;
          pulsestoempty = 0;
          positionstepper1 = finalpos1;
          positionstepper2 = finalpos2;
        }
        Serial.print("El motor 1 acaba en "); Serial.println(positionstepper1);
        Serial.print("Al motor 1 le quedan "); Serial.print(pulsestofill); Serial.println(" pulsos para llenar ");
        Serial.print("El motor 2 acaba en "); Serial.println(positionstepper2);
        Serial.print("Al motor 2 le quedan "); Serial.print(pulsestoempty); Serial.println(" pulsos para vaciar ");
        break;

      case EXTRACT:
        stepper2.setMaxSpeed(speedfill);
        stepper1.setMaxSpeed(speedempty);
        //        Serial.println(stepper1.speed());
        //        Serial.println(stepper2.speed());
        finalpos1 = positionstepper1 - pulsestoempty / stepstoadvance;
        finalpos2 = positionstepper2 + pulsestofill / stepstoadvance;
        if (finalpos2 > posmax || posmin > finalpos1)
        {
          Serial.println("There is not enought travel. Steppers will switch midway.");
        }
        Serial.println("Injecting stepper 2. Extracting stepper 1.");

        //Both stepper must go to the limit
        if (finalpos2 > posmax && posmin > finalpos1)
        {
          //          stepper1.setCurrentPosition(0);
          stepper2.move(-((posmax - positionstepper2) * stepspermL));
          //          stepper2.setCurrentPosition(0);
          stepper1.move((positionstepper1 - posmin ) * stepspermL);
          while (stepper2.distanceToGo() != 0 || stepper1.distanceToGo() != 0)
          {
            stepper2.run();
            stepper1.run();
            //            if (stepper1.run() == true)
            //            {
            //              contador++;
            //              Serial.print("Pulse "); Serial.print(contador); Serial.println(" of stepper 1 ");
            //            }
            //            if (stepper2.run() == true)
            //            {
            //              contador2++;
            //              Serial.print("Pulse "); Serial.print(contador2); Serial.println(" of stepper 2 ");
            //            }
          }
          Serial.print("El motor 2 se ha movido "); Serial.print((posmax - positionstepper2) * stepspermL); Serial.println(" pulsos ");
          Serial.print("El motor 1 se ha movido "); Serial.print((positionstepper1 - posmin) * stepspermL); Serial.println(" pulsos ");
          pulsestofill = pulsestofill - (posmax - positionstepper2) * stepspermL;
          pulsestoempty = pulsestoempty - (positionstepper1 - posmin) * stepspermL;
          positionstepper2 = posmax;
          positionstepper1 = posmin;
        }

        //Only stepper 2 move to max limit
        if (finalpos2 > posmax && finalpos1 >= posmin)
        {
          //          stepper1.setCurrentPosition(0);
          stepper2.move(-((posmax - positionstepper2) * stepspermL));
          //          stepper2.setCurrentPosition(0);
          stepper1.move(pulsestoempty);
          while (stepper2.distanceToGo() != 0 || stepper1.distanceToGo() != 0)
          {
            stepper2.run();
            stepper1.run();
            //            if (stepper1.run() == true)
            //            {
            //              contador++;
            //              Serial.print("Pulse "); Serial.print(contador); Serial.println(" of stepper 1 ");
            //            }
            //            if (stepper2.run() == true)
            //            {
            //              contador2++;
            //              Serial.print("Pulse "); Serial.print(contador2); Serial.println(" of stepper 2 ");
            //            }
          }
          Serial.print("El motor 2 se ha movido "); Serial.print((posmax - positionstepper2) * stepspermL); Serial.println(" pulsos ");
          Serial.print("El motor 1 se ha movido "); Serial.print(pulsestoempty); Serial.println(" pulsos ");
          pulsestofill = pulsestofill - (posmax - positionstepper2) * stepspermL;
          pulsestoempty = 0;
          positionstepper2 = posmax;
          positionstepper1 = finalpos1;
        }

        //Only stepper 1 move to max limit
        if (posmax >= finalpos2 && posmin > finalpos1)
        {
          //          stepper1.setCurrentPosition(0);
          stepper2.move(-pulsestofill);
          //          stepper2.setCurrentPosition(0);
          stepper1.move((positionstepper1 - posmin ) * stepspermL);
          while (stepper2.distanceToGo() != 0 || stepper1.distanceToGo() != 0)
          {
            stepper2.run();
            stepper1.run();
            //            if (stepper1.run() == true)
            //            {
            //              contador++;
            //              Serial.print("Pulse "); Serial.print(contador); Serial.println(" of stepper 1 ");
            //            }
            //            if (stepper2.run() == true)
            //            {
            //              contador2++;
            //              Serial.print("Pulse "); Serial.print(contador2); Serial.println(" of stepper 2 ");
            //            }
          }
          Serial.print("El motor 2 se ha movido "); Serial.print(pulsestofill); Serial.println(" pulsos ");
          Serial.print("El motor 1 se ha movido "); Serial.print((positionstepper1 - posmin) * stepspermL); Serial.println(" pulsos ");
          pulsestofill = 0;
          pulsestoempty = pulsestoempty - (positionstepper1 - posmin) * stepspermL;
          positionstepper2 = finalpos2;
          positionstepper1 = posmin;
        }
        //Both steppers inside range
        if (posmax >= finalpos2 && finalpos1 >= posmin)
        {
          //          stepper1.setCurrentPosition(0);
          stepper2.move(-pulsestofill);
          //          stepper2.setCurrentPosition(0);
          stepper1.move(pulsestoempty);
          while (stepper2.distanceToGo() != 0 || stepper1.distanceToGo() != 0)
          {
            stepper2.run();
            stepper1.run();
            //            if (stepper1.run() == true)
            //            {
            //              contador++;
            //              Serial.print("Pulse "); Serial.print(contador); Serial.println(" of stepper 1 ");
            //            }
            //            if (stepper2.run() == true)
            //            {
            //              contador2++;
            //              Serial.print("Pulse "); Serial.print(contador2); Serial.println(" of stepper 2 ");
            //            }
          }
          Serial.print("El motor 2 se ha movido "); Serial.print(pulsestofill); Serial.println(" pulsos ");
          Serial.print("El motor 1 se ha movido "); Serial.print(pulsestoempty); Serial.println(" pulsos ");
          pulsestofill = 0;
          pulsestoempty = 0;
          positionstepper2 = finalpos2;
          positionstepper1 = finalpos1;
        }
        Serial.print("El motor 2 acaba en "); Serial.println(positionstepper2);
        Serial.print("Al motor 2 le quedan "); Serial.print(pulsestofill); Serial.println(" pulsos para llenar ");
        Serial.print("El motor 1 acaba en "); Serial.println(positionstepper1);
        Serial.print("Al motor 1 le quedan "); Serial.print(pulsestoempty); Serial.println(" pulsos para vaciar ");
        break;
    }
    if (pulsestofill == 0 && pulsestoempty == 0)
    {
      delay(3000);
      for (int i = 0; i < PtC; i++)
      {
        if (newstatearray[i] == 1)
        {
          Serial.print("New value of "); Serial.print(xchange[i]); Serial.print(" "); Serial.print(ychange[i]); Serial.print(" is "); Serial.println(newstatearray[i]);
          screen[xchange[i]][ychange[i]] = 1;
          volumetotal = volumetotal + volume;
        }
        if (newstatearray[i] == 0)
        {
          Serial.print("New value of "); Serial.print(xchange[i]); Serial.print(" "); Serial.print(ychange[i]); Serial.print(" is "); Serial.println(newstatearray[i]);
          screen[xchange[i]][ychange[i]] = 0;
          volumetotal = volumetotal - volume;
        }
      }
      Serial.print(volumetotal); Serial.print(" mL of "); Serial.print(volumesyringe1 + volumesyringe2); Serial.println(" mL in the pixels");
      for (int i = 0; i < NPIXELS; i++) {
        if (servoStepper1[i].read() == OPEN)
        {
          servoStepper1[i].write(CLOSE);
        }
        if (servoStepper2[i].read() == OPEN)
        {
          servoStepper2[i].write(CLOSE);
        }
         if (servoStepper2[i].read() == OPEN29)
        {
          servoStepper2[i].write(CLOSE29);
        }
         if (servoStepper2[i].read() == OPEN25)
        {
          servoStepper2[i].write(CLOSE25);
        }
         if (servoStepper2[i].read() == OPEN27)
        {
          servoStepper2[i].write(CLOSE27);
        }
        
      }
      freeSTRINGS();
      state = stateREADCHANGES;
      return;
    }
    else
    {
      Serial.println("Changing steppers");
      delay(3000);
      CHOOSESTEPPER();
      MOVEVALVES();
    }
  }
}
